package kd.hrmp.hbjm.business.domain.service.impl;

import com.google.common.collect.Lists;
import kd.bos.dataentity.entity.DynamicObject;
import kd.bos.dataentity.resource.ResManager;
import kd.bos.entity.MainEntityType;
import kd.bos.orm.util.CollectionUtils;
import kd.bos.servicehelper.MetadataServiceHelper;
import kd.bos.threads.ThreadPool;
import kd.bos.threads.ThreadPools;
import kd.hr.hbp.common.constants.HRBaseConstants;
import kd.hrmp.hbjm.business.domain.repository.HBJMMsgRepository;
import kd.hrmp.hbjm.business.domain.service.ChangeMsgService;
import kd.hrmp.hbjm.business.task.PublishTask;
import kd.hrmp.hbjm.business.utils.JobUtils;
import kd.hrmp.hbjm.common.constants.ChangeEventConstants;
import kd.hrmp.hbjm.common.constants.HBJMConstants;
import kd.hrmp.hbjm.common.constants.PublishConstants;

import java.util.Arrays;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

public class ChangeMsgServiceImpl implements ChangeMsgService {
    private static ThreadPool threadPool = ThreadPools.newCachedThreadPool("HBJMChangeMsgServiceThreadpool",
        HRBaseConstants.INT_ONE, HRBaseConstants.INT_FOUR);

    private static ThreadLocal<List<DynamicObject>> msgDynThreadLocal = new ThreadLocal<>();

    @Override
    public void handleChangeMsg(List<DynamicObject> eventList, List<DynamicObject> jobs) {
        List<DynamicObject> msgList = saveChangeMsg(eventList, jobs);
        if (CollectionUtils.isEmpty(msgList)) {
            return;
        }
        msgDynThreadLocal.set(msgList);
    }

    private List<DynamicObject> saveChangeMsg(List<DynamicObject> eventList, List<DynamicObject> jobs) {
        if (jobs == null || jobs.size() == 0 || eventList == null || eventList.size() == 0) {
            return null;
        }
        Map<Long, DynamicObject> jobToEventMap =
            eventList.stream().collect(Collectors.toMap(event -> event.getLong("targetjob"), event -> event));
        MainEntityType dataEntityType = MetadataServiceHelper.getDataEntityType(PublishConstants.JOBCHANGEMSG_PAGE);
        List<DynamicObject> msgList = Lists.newArrayListWithExpectedSize(jobs.size());
        for (DynamicObject job : jobs) {
            // 变动类型为 新增、变更、禁用的 才发消息（初始化不发消息）
            long changetype = job.getLong("changetype");
            if (changetype == ChangeEventConstants.TYPE_ADD || changetype == ChangeEventConstants.TYPE_UPDATE
                || changetype == ChangeEventConstants.TYPE_DISABLE) {
                DynamicObject msg = new DynamicObject(dataEntityType);
                DynamicObject event = jobToEventMap.get(job.getLong("boid"));
                setMsgValue(msg, job, event);
                msgList.add(msg);
            }
        }

        JobUtils.callOp("save",msgList.toArray(new DynamicObject[msgList.size()]));
        return msgList;
    }

    @Override
    public void sendMsg() {
        List<DynamicObject> msgList = msgDynThreadLocal.get();
        if (CollectionUtils.isEmpty(msgList)) {
            return;
        }
        // 这里等保存完，再查出来，再进行消息的发送。 防止后面保存的时候，update变成insert，报id重复
        List<Long> ids = msgList.stream().map(msg -> msg.getLong(HRBaseConstants.ID)).collect(Collectors.toList());
        Map<Long, DynamicObject> idToJobMap =  msgList.stream()
                .collect(Collectors.toMap(msg -> msg.getLong(HRBaseConstants.ID), msg -> msg.getDynamicObject("job")));
        DynamicObject[] msgs = HBJMMsgRepository.getInstance().queryRetryMsgsById(ids);
        for(DynamicObject dyn : msgs){
            DynamicObject job = idToJobMap.get(dyn.getLong(HRBaseConstants.ID));
            if(job != null){
                dyn.set("job", job);
            }
        }
        threadPool.execute(new PublishTask(Arrays.asList(msgs)));
        msgDynThreadLocal.remove();
    }

    @Override
    public void retrySend(DynamicObject[] needRetrysendMsgs) {
        PublishTask publishTask = new PublishTask();
        publishTask.publish(Arrays.asList(needRetrysendMsgs), true);
    }

    /**
     * @param boIds
     * @description: 根据职位删除消息
     * @return: void
     * @author rd_ningjie
     * @date: 2022/7/25 18:48
     */
    @Override
    public void delByJobBoids(List<Long> josDataIds) {
        // 根据boid查询所有的dataid
        HBJMMsgRepository.getInstance().delByJobIds(josDataIds);
    }

    private void setMsgValue(DynamicObject msg, DynamicObject job, DynamicObject event) {
        msg.set("number", event.getString("event"));
        msg.set("createorg", job.getDynamicObject("createorg"));
        msg.set("org", job.getDynamicObject("org"));
        msg.set("useorg", job.getDynamicObject("useorg"));
        msg.set("ctrlstrategy", job.getString("ctrlstrategy"));
        msg.set("changedesc", job.getString("changedesc2"));
        msg.set("effetdate", event.getDate("changedate"));
        msg.set("changetype", event.getString("changetype"));
        msg.set("changescene", event.getString("changescene"));
        msg.set("changesubscene", event.getDynamicObjectCollection("changesubscene"));
        long changeoperate = job.getLong("changeoperate");
        msg.set("changeoperate", changeoperate);
        msg.set("changereason", event.getString("changereason"));
        msg.set("changeuser", job.getString("modifier_id"));
        msg.set("changetime", event.getDate("createtime"));
        msg.set("job", job);
        msg.set("sendstate", PublishConstants.PUBLISH_WAIT);
        msg.set("status", 'C');
        msg.set("enable", HRBaseConstants.ENABLED);
        setMsgPublishInfo(changeoperate, job, msg);
    }

    private void setMsgPublishInfo(long changeoperate, DynamicObject job, DynamicObject msg) {
        String title;
        String tag;
        String desc;
        if (changeoperate == ChangeEventConstants.ADD) {
            title = String.format(
                ResManager.loadKDString("新增职位（编码：%1$s， 名称：%2$s）", "ChangeEventServiceImpl_0",
                    HBJMConstants.HRMP_HBJM_BUSINESS),
                job.getString(HRBaseConstants.NUMBER), job.getString(HRBaseConstants.NAME));
            tag = ResManager.loadKDString("职位新增", "ChangeEventServiceImpl_3", HBJMConstants.HRMP_HBJM_BUSINESS);
            desc = ResManager.loadKDString("职位新增通知", "ChangeEventServiceImpl_6", HBJMConstants.HRMP_HBJM_BUSINESS);

        } else if (changeoperate == ChangeEventConstants.UPDATE) {
            title = String.format(
                ResManager.loadKDString("变更职位（编码：%1$s， 名称：%2$s）", "ChangeEventServiceImpl_1",
                    HBJMConstants.HRMP_HBJM_BUSINESS),
                job.getString(HRBaseConstants.NUMBER), job.getString(HRBaseConstants.NAME));
            tag = ResManager.loadKDString("职位变更", "ChangeEventServiceImpl_4", HBJMConstants.HRMP_HBJM_BUSINESS);
            desc = ResManager.loadKDString("职位变更通知", "ChangeEventServiceImpl_7", HBJMConstants.HRMP_HBJM_BUSINESS);
        } else if (changeoperate == ChangeEventConstants.DISABLE) {
            title = String.format(
                ResManager.loadKDString("禁用职位（编码：%1$s， 名称：%2$s）", "ChangeEventServiceImpl_2",
                    HBJMConstants.HRMP_HBJM_BUSINESS),
                job.getString(HRBaseConstants.NUMBER), job.getString(HRBaseConstants.NAME));
            tag = ResManager.loadKDString("职位禁用", "ChangeEventServiceImpl_5", HBJMConstants.HRMP_HBJM_BUSINESS);
            desc = ResManager.loadKDString("职位禁用通知", "ChangeEventServiceImpl_8", HBJMConstants.HRMP_HBJM_BUSINESS);
        } else {
            title = "";
            tag = "";
            desc = "";
        }
        msg.set("msgtag", tag);
        msg.set("msgtitle", title.length() > PublishConstants.TITLE_LENGTHLIMIT
            ? title.substring(0, PublishConstants.TITLE_LENGTHLIMIT) : title);
        msg.set("msgdesc", desc);
    }
}
